library(dplyr)
library(Hmisc)

art <- read.csv("data/art.csv", stringsAsFactors = FALSE, na.strings = c(NA, ''))
basic <- read.csv("data/basic.csv", stringsAsFactors = FALSE, na.strings = c(NA, ''))
ce <- read.csv("data/ce.csv", stringsAsFactors = FALSE, na.strings = c(NA, ''))
ce_cancer <- read.csv("data/ce_cancer.csv", stringsAsFactors = FALSE, na.strings = c(NA, ''))
follow <- read.csv("data/follow.csv", stringsAsFactors = FALSE, na.strings = c(NA, ''))
cd4 <- read.csv("data/lab_cd4.csv", stringsAsFactors = FALSE, na.strings = c(NA, ''))
rna <- read.csv("data/lab_rna.csv", stringsAsFactors = FALSE, na.strings = c(NA, ''))
visit <- read.csv("data/visit.csv", stringsAsFactors = FALSE, na.strings = c(NA, ''))

coorte_art <- read.csv("data/coorte/art.csv")
coorte_bas <- read.csv("data/coorte/basic.csv")
coorte_ce <- read.csv("data/coorte/ce.csv")
coorte_ce_cancer <- read.csv("data/coorte/ce_cancer.csv")
coorte_follow <- read.csv("data/coorte/follow.csv")
coorte_cd4 <- read.csv("data/coorte/lab_cd4.csv")
coorte_rna <- read.csv("data/coorte/lab_rna.csv")

#subset data
dat <- subset(basic, site != "haiti")

#enrollment date
dat <- subset(dat, baseline_d >= '2000-01-01' & baseline_d <= '2018-12-31')
# 23231 people after subset

#remove duplicated patients from INI (IPEC) sites, if both in ccasanet and coorte then keep ccasanet info
ids <- read.csv("data/COORTE and CCASAnet ID matching file.csv")
colnames(ids) <- c('coorte_id', 'patient_id')
dupsites <- subset(ids, patient_id %in% dat$patient_id)
#1033 people in both coorte and ccasanet
coorte_bas2 <- subset(coorte_bas, !(patidn %in% dupsites$coorte_id))

#format coorte data to match basic ccasanet format
formatcols <- c("patient", "site", "center", "mode", "mode_oth", "birth_d", "enrol_d", "art_d", "hivdiagnosis_d", "baseline_d")
coorte_bas2[formatcols] <- lapply(coorte_bas2[formatcols], as.character)
names(coorte_bas2)[names(coorte_bas2)=='edu_lvl'] <- 'school_lvl'
names(coorte_bas2)[names(coorte_bas2)=='art_d'] <- 'art_sd'

dat <- dplyr::bind_rows(dat, coorte_bas2)
#don't need to subset coorte baseline dates, they are already from 2000 to 2016

#age 
dat$baseline_d <- as.Date(dat$baseline_d, origin= '1970-01-01')
dat$birth_d <- as.Date(dat$birth_d, origin= '1970-01-01')
dat$age_enroll <- as.numeric(dat$baseline_d - dat$birth_d)/365.25
# basic_under18 <- subset(dat, age_enroll < 16) #there are 2312 people under 16
dat <- subset(dat, age_enroll >= 16)

#ks
#merge in ks date of diagnosis
coorte_ce_cancer[] <- lapply(coorte_ce_cancer[], as.character)
coorte_ce_cancer$cancer_d <- as.Date(coorte_ce_cancer$cancer_d, origin='1970-01-01')
ce_cancer$cancer_d <- as.Date(ce_cancer$cancer_d, origin='1970-01-01')
ce_cancer <- bind_rows(ce_cancer, coorte_ce_cancer)

#First cancer date & type
#If KS is someone's second or third cancer, date of that cancer needs to be attached separately 
ce_cancer$cancer_d <- as.Date(ce_cancer$cancer_d, origin= '1970-01-01')
min_cancer <- ce_cancer %>% group_by(patient) %>% slice(which.min(cancer_d))
min_cancer <- as.data.frame(min_cancer)
dat$cancer_type <- min_cancer[match(dat$patient, min_cancer$patient, nomatch= NA), 'cancer_type']
dat$cancer_d <- min_cancer[match(dat$patient, min_cancer$patient, nomatch= NA), 'cancer_d']
label(dat$cancer_d) <- "Date of first cancer diagnosis"
label(dat$cancer_type) <- "Cancer type"

#Create a separate data containing only KS info
ce_cancer_ks <- subset(ce_cancer, cancer_type=='KS')
#Keep only info of first KS
ce_cancer_ks <- ce_cancer_ks %>% group_by(patient) %>% slice(which.min(cancer_d))
ce_cancer_ks <- as.data.frame(ce_cancer_ks)
dat$ks_diagnosis_d <- ce_cancer_ks[match(dat$patient, ce_cancer_ks$patient), c('cancer_d')]
dat$ks <- ifelse(!is.na(dat$ks_diagnosis_d), 1, 0)
dat$ks.factor <- factor(dat$ks, levels=c(0:1), labels= c('No KS', 'KS'))

# Exclude KS case in 2017
dat <- subset(dat, ks_diagnosis_d <= "2017-12-31" | is.na(ks_diagnosis_d)) #to 27425

# How many people have KS before enrollment?
# ks_before_enr <- subset(dat, ks_diagnosis_d < baseline_d)
# describe(as.numeric(ks_before_enr$baseline_d - ks_before_enr$ks_diagnosis_d))
# A total of 257 people had KS before enrollment and the median difference in days is 43

# to see who has other ADE cancers that are not KS
# describe(dat[!is.na(dat[, 'ks_diagnosis_d']), 'cancer_type'])
# 7 people have ade cancers in addition to KS
# dat[which(!is.na(dat$ks_diagnosis_d) & dat$cancer_type %in% c('KS', 'NHL', 'ICC')), c('patient', 'ks_diagnosis_d', 'cancer_d', 'cancer_type')]

#Add history of *non-cancer* ADE before enrollment and first haart
ce <- ce %>% mutate(ade_type= case_when(grepl('^ade', ce_id) ~ 'ADE', 
                                        grepl('^nade', ce_id) ~ 'NADE'))
ce$ce_d <- as.Date(ce$ce_d, origin= '1970-01-01')

# #to check if ade cancer date is the date of KS diagnosis; if KS was used to define ADE on that day
# ade_cancer_ks <- subset(ce, ce_id=='ade_cancer' & ce_oth=='KS')
#Compare ade_cancer KS diagnosis dates with KS diagnosis dates matched in from ce_cancer table
# dat$ks_diagnosis_d2 <- ade_cancer_ks[match(dat$patient, ade_cancer_ks$patient), 'ce_d']
# dat[which(dat$ks_diagnosis_d != dat$ks_diagnosis_d2), c('patient', 'ks_diagnosis_d', 'ks_diagnosis_d2', 'cancer_type')]

#Exclude ADE cancers and NADEs
ce_min <- subset(ce, ce_id!= 'ade_cancer' & ade_type=='ADE')
ce_min <- ce_min %>% group_by(patient) %>% filter(ade_type=='ADE') %>% slice(which.min(ce_d))
ce_min <- as.data.frame(ce_min)

dat$min_ce_d <- ce_min[match(dat$patient, ce_min$patient), 'ce_d']
dat$ade_type <- ce_min[match(dat$patient, ce_min$patient), 'ce_id']
dat$ade_before_enr <- ifelse(dat$min_ce_d < dat$baseline_d, 'Yes', 'No')
dat$ade_before_enr <- ifelse(is.na(dat$ade_before_enr), 'No', dat$ade_before_enr)


#exclude those with KS dx prior to entry 
ks_prior <- subset(dat, cancer_type=='KS' & (cancer_d < baseline_d))
dat <- subset(dat, !(dat$patient %in% ks_prior$patient))
# 257 excluded

#Closing date
coorte_follow[c("patient", "site", "center", "patidn", "death_d", "l_alive_d", "lastobs_d")] <- lapply(coorte_follow[c("patient", "site", "center", "patidn", "death_d", "l_alive_d", "lastobs_d")], as.character)
follow <- bind_rows(follow, coorte_follow)

dat$l_alive_d <- follow[match(dat$patient, follow$patient, nomatch = NA), 'l_alive_d']
dat$l_alive_d <- as.Date(dat$l_alive_d, origin= '1970-01-01')
dat$lastobs_d <- follow[match(dat$patient, follow$patient, nomatch = NA), 'lastobs_d']
dat$lastobs_d <- as.Date(dat$lastobs_d, origin= '1970-01-01')

dat <- dat %>% group_by(site) %>% mutate(closing_d= case_when(site== "argentina"~ "2018-12-31",
                                                              site== "brazil"~ "2018-12-31",
                                                              site== "chile"~ "2016-12-31",
                                                              site== "coorte"~ "2014-12-31",
                                                              center=="hn-he"~ "2017-12-31",
                                                              center=="hn-ss"~ "2015-12-31",
                                                              site=="mexico"~ "2018-12-31",
                                                              site=="peru"~ "2018-12-31")) %>% ungroup()
dat <- as.data.frame(dat)
dat$closing_d <- as.Date(dat$closing_d)
dat$enrol_d <- as.Date(dat$enrol_d)

# need to remove those who enrolled after closing date
dat <- subset(dat, enrol_d < closing_d) #25981 total

# follow-up should be cutoff to closing date if beyond that 
dat$l_alive_d <- ifelse(dat$l_alive_d > dat$closing_d, dat$closing_d, dat$l_alive_d)
dat$l_alive_d <- as.Date(dat$l_alive_d, origin="1970-01-01")

#LTFU defined as no clinic visit for a year
dat$death_y <- follow[match(dat$patient, follow$patient, nomatch = NA), 'death_y']
dat$ltfu <- NA
dat$ltfu[dat$death_y==1] <- 0
dat$ltfu[dat$death_y==0 & (dat$closing_d - dat$l_alive_d) > 365] <- 1
dat$ltfu[dat$death_y==0 & (dat$closing_d - dat$l_alive_d) <= 365] <- 0  
dat$ltfu.factor <- factor(dat$ltfu, levels= c(0:1), labels= c('No', 'Yes'))
label(dat$ltfu.factor) <- "Lost to follow-up"

#Attach date of death
dat$death_d <- follow[match(dat$patient, follow$patient, nomatch = NA), 'death_d']
dat$death_d <- as.Date(as.character(dat$death_d),"%Y-%m-%d")

#Sex and mode combined
dat$sex_risk <- NA
dat$sex_risk[dat$male_y==0 & dat$mode=='Bisexual'] <- "Heterosexual women"
dat$sex_risk[dat$male_y==1 & dat$mode=='Bisexual'] <- "MSM"
dat$sex_risk[dat$male_y==0 & dat$mode=='Generic Sexual'] <- "Heterosexual women"
dat$sex_risk[dat$male_y==1 & dat$mode=='Generic Sexual'] <- "Heterosexual men"
dat$sex_risk[dat$male_y==1 & dat$mode=='Heterosexual contact and Injecting drug user'] <- "Heterosexual men"
dat$sex_risk[dat$male_y==0 & dat$mode=='Heterosexual contact and Injecting drug user'] <- "Heterosexual women"
dat$sex_risk[dat$male_y==1 & dat$mode=='Homo/Bisexual and Injecting drug user'] <- "MSM"
dat$sex_risk[dat$male_y==0 & dat$mode=='Homo/Bisexual and Injecting drug user'] <- "Other/Unknown women"
dat$sex_risk[dat$male_y==0 & dat$mode=='Homosexual contact'] <- "Other/Unknown women"
dat$sex_risk[dat$male_y==1 & dat$mode=='Homosexual contact'] <- "MSM"
dat$sex_risk[dat$male_y==0 & dat$mode== 'Injecting drug user'] <- "Other/Unknown women"
dat$sex_risk[dat$male_y==1 & dat$mode== 'Injecting drug user'] <- "Other/Unknown men"
dat$sex_risk[dat$male_y==0 & dat$mode== 'Other (specify in mode_oth)'] <- "Other/Unknown women"
dat$sex_risk[dat$male_y==1 & dat$mode== 'Other (specify in mode_oth)'] <- "Other/Unknown men"
dat$sex_risk[dat$male_y==0 & dat$mode== 'Perinatal'] <- "Other/Unknown women"
dat$sex_risk[dat$male_y==1 & dat$mode== 'Perinatal'] <- "Other/Unknown men"
dat$sex_risk[dat$male_y==0 & dat$mode=='Transfusion nonhemophilia related'] <- "Other/Unknown women"
dat$sex_risk[dat$male_y==1 & dat$mode=='Transfusion nonhemophilia related'] <- "Other/Unknown men"
dat$sex_risk[dat$mode== 'Unknown' & dat$male_y==0] <- "Other/Unknown women"
dat$sex_risk[dat$mode== 'Unknown' & dat$male_y==1] <- "Other/Unknown men"
dat$sex_risk[dat$male_y==0 & dat$mode== 'Hemophiliac'] <- "Other/Unknown women"
dat$sex_risk[dat$male_y==1 & dat$mode== 'Hemophiliac'] <- "Other/Unknown men"
dat$sex_risk[is.na(dat$mode) & dat$male_y==0] <- "Other/Unknown women"
dat$sex_risk[is.na(dat$mode) & dat$male_y==1] <- "Other/Unknown men"
dat$sex_risk[dat$male_y==0 & dat$mode=='Heterosexual contact'] <- 'Heterosexual women'
dat$sex_risk[dat$male_y==1 & dat$mode=='Heterosexual contact'] <- 'Heterosexual men'
dat$sex_risk[dat$male_y==0 & dat$mode=='Other (specify in mode_oth)'] <- "Other/Unknown women"
dat$sex_risk[dat$male_y==1 & dat$mode=='Other (specify in mode_oth)'] <- "Other/Unknown men"
dat$sex_risk <- ifelse(is.na(dat$sex_risk) & dat$male_y==0, "Other/Unknown women", dat$sex_risk)
dat$sex_risk <- ifelse(is.na(dat$sex_risk) & dat$male_y==1, "Other/Unknown men", dat$sex_risk)
dat$sex_risk <- as.factor(dat$sex_risk)

#IDU
dat$mode <- ifelse(is.na(dat$mode), "Unknown", dat$mode)
dat$mode <- as.factor(dat$mode)

dat$idu <- NA
dat$idu <- ifelse(dat$mode=='Injecting drug user' | 
                    dat$mode=='Heterosexual contact and Injecting drug user' | 
                    dat$mode== 'Homo/Bisexual and Injecting drug user', 1, 0)

dat$idu.factor <- factor(dat$idu, levels=c(0:1), labels= c("No", "Yes"))
label(dat$idu.factor) <- "Injecting drug user"

#Year of KS dx
dat$ks_year <- substr(dat$ks_diagnosis_d, 1, 4)
dat$ks_year <- as.numeric(dat$ks_year)
label(dat$ks_year) <- "Year of KS diagnosis"

#first HAART
#match in art class for coorte
coorte_art$art_class <- art[match(coorte_art$art_id, art$art_id, nomatch = NA), 'art_class']
coorte_art[c("patient", "site", "center", "art_id", "art_sd", "art_ed")] <- lapply(coorte_art[c("patient", "site", "center", "art_id", "art_sd", "art_ed")], as.character)
coorte_art2 <- subset(coorte_art, select= c(patient, site, center, patidn, art_sd, art_ed, art_class))
names(coorte_art2)[names(coorte_art2)=='patidn'] <- 'patient_id'


art2 <- subset(art, select= c(patient, site, center, patient_id, art_sd, art_ed, art_class))
art_new <- bind_rows(art2, coorte_art2)
art_new$art_sd <- as.Date(art_new$art_sd, origin= '1970-01-01')
art_new <- subset(art_new, !is.na(art_sd))
art_new <- art_new %>% group_by(patient_id) %>% mutate(min_haart_sd= min(art_sd))
art_min <- art_new %>% group_by(patient_id) %>% slice(which.min(art_sd))
art_min <- as.data.frame(art_min)
art_new <- as.data.frame(art_new)


dat$art_class <- art_min[match(dat$patient, art_min$patient), 'art_class']
dat$min_haart_sd <- art_new[match(dat$patient, art_new$patient), 'min_haart_sd']
dat$min_haart_sd <- as.Date(dat$min_haart_sd, origin= '1970-01-01')
dat$age_haart <- (as.numeric(dat$min_haart_sd) - as.numeric(dat$birth_d))/ 365.25
dat$haart_year <- as.numeric(substr(dat$min_haart_sd, 1, 4))
label(dat$haart_year) <- "Year of first ART"

#ade before ART
dat$ade_before_art <- ifelse(dat$min_ce_d < dat$min_haart_sd, 'Yes', 'No')
dat$ade_before_art <- ifelse(is.na(dat$ade_before_art), 'No', dat$ade_before_art)
label(dat$ade_before_enr) <- "History of ADE before enroll"
label(dat$ade_before_art) <- "History of ADE before ART"
label(dat$ade_type) <- "ADE type"


#Attach CD4 at ART initiation
#-180 to +30 days from date of ART initiation
dat <- as.data.frame(dat)
cd4 <- subset(cd4, select= c(patient, cd4_d, cd4_v, site))
coorte_cd4 <- subset(coorte_cd4, select= c(patient, cd4_d, cd4_v, site))
coorte_cd4[c("patient", "site")] <- lapply(coorte_cd4[c("patient", "site")], as.character)
# coorte_cd4$cd4_d <- as.Date(coorte_cd4$cd4_d)
cd4 <- bind_rows(cd4, coorte_cd4)

cd4$min_haart_sd <- dat[match(cd4$patient, dat$patient), 'min_haart_sd']
cd4$cd4_d <- as.Date(cd4$cd4_d, origin= '1970-01-01')
cd4_haart <- cd4 %>% group_by(patient) %>% filter((cd4_d - min_haart_sd >= -180) & (cd4_d - min_haart_sd <= 30)) %>% 
  mutate(cd4_minus_haart = cd4_d - min_haart_sd) %>% slice(which(abs(cd4_minus_haart) == min(abs(cd4_minus_haart))))
cd4_haart<- as.data.frame(cd4_haart)
cd4_haart_unique <- cd4_haart %>% group_by(patient) %>% slice(which.min(cd4_minus_haart))
cd4_haart_unique <- as.data.frame(cd4_haart_unique)
dat$cd4_haart <- cd4_haart_unique[match(dat$patient, cd4_haart_unique$patient), 'cd4_v']

# Attach RNA at ART 
rna <- subset(rna, select=c(patient, rna_d, rna_v, site))
coorte_rna <- subset(coorte_rna, select = c(patient, rna_d, rna_v, site))
coorte_rna$rna_d <- as.Date(coorte_rna$rna_d)
coorte_rna[c("patient", "site")] <- lapply(coorte_rna[c("patient", "site")], as.character)
rna$rna_d <- as.Date(rna$rna_d)
rna <- bind_rows(rna, coorte_rna)
rna$min_haart_sd <- dat[match(rna$patient, dat$patient), 'min_haart_sd']
rna_haart <- rna %>% group_by(patient) %>% filter((rna_d - min_haart_sd >= -180) & (rna_d - min_haart_sd <=30)) %>%
  mutate(rna_minus_haart= rna_d - min_haart_sd) %>% slice(which(abs(rna_minus_haart) == min(abs(rna_minus_haart))))
rna_haart <- as.data.frame(rna_haart)
rna_haart_unique <- rna_haart %>% group_by(patient) %>% slice(which.min(rna_minus_haart)) %>% ungroup %>% as.data.frame()
dat$rna_haart <- rna_haart_unique[match(dat$patient, rna_haart_unique$patient), "rna_v"]

#Age at KS dx
dat$age_ks <- ifelse(!is.na(dat$ks_diagnosis_d), as.numeric(dat$ks_diagnosis_d - dat$birth_d)/365.25, NA)

#Label Vars and clean-up
dat$enrol_d <- as.Date(dat$enrol_d, origin= '1970-01-01')
label(dat$age_enroll) <- "Age at enrollment"
label(dat$age_ks) <- "Age at KS diagnosis"
label(dat$sex_risk) <- "Sex/Route of Infection"
label(dat$cancer_d) <- "Date of cancer diagnosis"
label(dat$min_haart_sd) <- "Date of first ART"
label(dat$age_haart) <- "Age at first ART"
label(dat$cd4_haart) <- "CD4 at first ART"

#site clean-up
dat$country <- ifelse(dat$site=='coorte', 'BRA', dat$country)
dat$country.factor <- factor(dat$country, levels= c('ARG', 'BRA', 'CHL', 'HND', 'MEX', 'PER'), labels= c('Argentina', 'Brazil', 'Chile', 'Honduras', 'Mexico', 'Peru'))
label(dat$country.factor) <- "Country"

dat$site.factor <- factor(dat$site, levels= c('brazil', 'chile', 'coorte','honduras', 'mexico', 'peru'), labels= c('Brazil', 'Chile', 'Coorte','Honduras', 'Mexico', 'Peru'))
label(dat$site.factor) <- "Site"

#Cancer minus enrollment
dat$cancerminusenroll <- as.numeric(dat$cancer_d - dat$enrol_d)

#death yes
dat$death_y.factor <- factor(dat$death_y, levels=c(0:1), labels=c('Alive', 'Dead'))
label(dat$death_y.factor) <- "Mortality status"


#Aim 3 only KS patients
ks <- subset(dat, ks.factor== "KS")
ks_ade <- subset(ks, ade_type=='ade_cancer')
ks_ade_cancer <- ks_ade[which(ks_ade$min_ce_d== ks_ade$cancer_d), c('patient', 'cancer_type', 'ade_type', 'cancer_d', 'min_ce_d', 'baseline_d', 'min_haart_sd')]

#attach CD4 dates use 180 to 30 for now
#-180 to +30 days from date of dx
cd4$ks_diagnosis_d <- dat[match(cd4$patient, dat$patient), 'ks_diagnosis_d']
cd4_ksdx <- cd4 %>% group_by(patient) %>% filter((cd4_d - ks_diagnosis_d >= -180) & (cd4_d - ks_diagnosis_d <= 30)) %>% 
  mutate(cd4_minus_dx= cd4_d - ks_diagnosis_d) %>% slice(which(abs(cd4_minus_dx) == min(abs(cd4_minus_dx)))) %>% slice(which.min(cd4_minus_dx))
cd4_ksdx <- as.data.frame(cd4_ksdx)
ks$cd4_ksdx <- cd4_ksdx[match(ks$patient, cd4_ksdx$patient), 'cd4_v']

#attach RNA dates use 180 to 30 
rna$ks_diagnosis_d <- dat[match(rna$patient, dat$patient), 'ks_diagnosis_d']
rna$rna_d <- as.Date(rna$rna_d, origin= '1970-01-01')
rna_ksdx <- rna %>% group_by(patient) %>% filter((rna_d - ks_diagnosis_d >= -180) & (rna_d - ks_diagnosis_d <= 30)) %>% 
  mutate(rna_minus_dx= rna_d - ks_diagnosis_d) %>% slice(which(abs(rna_minus_dx) == min(abs(rna_minus_dx)))) %>% slice(which.min(rna_minus_dx))
rna_ksdx <- as.data.frame(rna_ksdx)
ks$rna_ksdx <- rna_ksdx[match(ks$patient, rna_ksdx$patient), "rna_v"]

label(ks$cd4_ksdx) <- "CD4 at KS diagnosis"


#Follow-up time til death or censor
# ks$follow_death <- ifelse(ks$death_y==0 & ks$ltfu==0, as.numeric(ks$closing_d - ks$ks_diagnosis_d), (ks$l_alive_d - ks$ks_diagnosis_d))
ks$follow_death <- ifelse(ks$death_y==1, as.numeric(ks$death_d - ks$ks_diagnosis_d), (ks$l_alive_d - ks$ks_diagnosis_d))
label(ks$follow_death) <- "Follow-up time in days until death or censoring"

#HAART before KS or after
ks$ks_before_art <- ifelse(ks$min_haart_sd < ks$ks_diagnosis_d, "KS after ART", "KS before ART")
ks$ks_before_art <- ifelse(is.na(ks$min_haart_sd), "KS before ART", ks$ks_before_art)
ks$ks_before_art <- as.factor(ks$ks_before_art)
label(ks$ks_before_art) <- "KS before ART"

ks$ks_before_art_num <- ifelse(ks$ks_before_art=='KS before ART', 1, 0)

ks$rna_ksdx.noneg <- ifelse(ks$rna_ksdx < 400, 400, ks$rna_ksdx)
ks$log_rna_ksdx.noneg <- log10(ks$rna_ksdx.noneg)
label(ks$rna_ksdx.noneg) <- "RNA at KS diagnosis"
label(ks$log_rna_ksdx.noneg) <- "Log10 VL at KS diagnosis"

# save('dat', 'ks', file= 'Aim3.Rda')

